 /*
  * Khoros: $Id: dlsynth.c,v 1.2 1992/03/20 23:32:02 dkhoros Exp $
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: dlsynth.c,v 1.2 1992/03/20 23:32:02 dkhoros Exp $";
#endif

 /*
  * $Log: dlsynth.c,v $
 * Revision 1.2  1992/03/20  23:32:02  dkhoros
 * VirtualPatch5
 *
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: dlsynth.c
 >>>>
 >>>>      Program Name: dlsynth
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	1D Lattice Filter Synthesis
 >>>>	
 >>>>	

 >>>>            Author: Ramiro Jordan
 >>>>
 >>>> Date Last Updated: Mon Feb 17 11:20:38 1992
 >>>>
 >>>>          Routines: main- the main program for dlsynth
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "dlsynth.h"


/****************************************************************
 *
 * Routine Name:  main program for dlsynth
 *
 *       Input:  
 *         -i1  input parameter file (xviff file) 
 *         -i2  input driving file (xviff file) 
 *         -o  resulting synthesized output file (xviff file) 
 *
 *
 *         [-j] scalar processing (0) or vector processing (1) (default)
 *         [-d] down vectors(0) or across bands(1) (default)
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
struct xvimage *lattice, *driver, *readimage();
int    arith_type,procdir;
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
if (check_args()) exit(1);
READIMAGE(lattice,driver);
CHECKTYPE(program,lattice,driver);
CHECKINPUT(program,lattice,driver);

arith_type = dlsynth->j_logic;
procdir    = dlsynth->d_logic;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! ldlsynth ( lattice, driver, arith_type, procdir ) )
        {
            (void) fprintf(stderr, "ldlsynth Failed\n");
            exit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
    writeimage(dlsynth->o_file,driver);
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for dlsynth 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "dlsynth :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t1D Lattice Filter Synthesis\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i1  input parameter file (xviff file)  (infile)\n");
	fprintf(stderr,"\t-i2  input driving file (xviff file)  (infile)\n");
	fprintf(stderr,"\t-o  resulting synthesized output file (xviff file)  (outfile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-j]  scalar processing (0) or vector processing (1) (default) (boolean) [true]\n");
	fprintf(stderr,"\t[-d]  down vectors(0) or across bands(1) (default) (boolean) [true]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for dlsynth\n");
	fprintf(stderr, "\t[-U] Gives the usage for dlsynth\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called dlsynth.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses dlsynth.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for dlsynth
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i1_pstr = 
	"\nEnter: (i1) input parameter file (xviff file) \n        {infile}: "; 
char	*i2_pstr = 
	"\nEnter: (i2) input driving file (xviff file) \n        {infile}: "; 
char	*o_pstr = 
	"\nEnter: (o) resulting synthesized output file (xviff file) \n        {outfile}: "; 
char	*j_pstr = 
	"\nEnter: (j) scalar processing (0) or vector processing (1) (default) \n        {boolean, (y/n)  [yes] }: "; 
char	*d_pstr = 
	"\nEnter: (d) down vectors(0) or across bands(1) (default) \n        {boolean, (y/n)  [yes] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "dlsynth", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "dlsynth.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "dlsynth: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "dlsynth.ans");
		}

	     }

	}

	/*
	 * allocate the dlsynth structure
	 */
	dlsynth = (dlsynth_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (dlsynth_struct));

	/*
	 * get required arguments for dlsynth
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(dlsynth->i1_flag = vget_infile(fid, prompt_flag,
			"-i1", i1_pstr, &(dlsynth->i1_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(dlsynth->i1_flag))
		{
		    fprintf(stderr, "\t\ti1 is required, please re-enter: \n");
		    dlsynth->i1_flag = vget_infile(fid, prompt_flag,
		    "-i1", i1_pstr, &(dlsynth->i1_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i1' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(dlsynth->i2_flag = vget_infile(fid, prompt_flag,
			"-i2", i2_pstr, &(dlsynth->i2_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(dlsynth->i2_flag))
		{
		    fprintf(stderr, "\t\ti2 is required, please re-enter: \n");
		    dlsynth->i2_flag = vget_infile(fid, prompt_flag,
		    "-i2", i2_pstr, &(dlsynth->i2_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i2' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(dlsynth->o_flag = vget_outfile(fid, prompt_flag,
			"-o", o_pstr, &(dlsynth->o_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(dlsynth->o_flag))
		{
		    fprintf(stderr, "\t\to is required, please re-enter: \n");
		    dlsynth->o_flag = vget_outfile(fid, prompt_flag,
		    "-o", o_pstr, &(dlsynth->o_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-o' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for dlsynth
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	dlsynth->j_flag = vget_logic(fid, prompt_flag,
	"-j", j_pstr, &(dlsynth->j_logic), 1);

	dlsynth->d_flag = vget_logic(fid, prompt_flag,
	"-d", d_pstr, &(dlsynth->d_logic), 1);


}



